unit WABD_ISAPI;

interface

uses Windows, SysUtils, Classes, Forms, StdCtrls, ISAPI;

const
   WABD_VersionName = 'Web Application Builder for Delphi v1.00';

type
   TServerVar = record
      REMOTE_ADDR       : string;
      REMOTE_HOST       : string;
      REMOTE_USER       : string;
      SERVER_NAME       : string;
      SERVER_PORT       : string;
      SERVER_PROTOCOL   : string;
      SERVER_SOFTWARE   : string;
      SCRIPT_NAME       : string;
   end;

   TWABD_Callback = function(var ECB: TEXTENSION_CONTROL_BLOCK):string of object;

function RunningLocal: boolean;

// These functions are required to interface with ISAPI
function GetExtensionVersion(var Ver: THSE_VERSION_INFO):boolean; stdcall;
function HttpExtensionProc( var ECB: TEXTENSION_CONTROL_BLOCK ):DWORD; stdcall;

// These are ISAPI Help functions
function  ISAPI_GetOneVar(var ECB: TEXTENSION_CONTROL_BLOCK; const VarName: string): string;
procedure ISAPI_GetServerVars(var ECB: TEXTENSION_CONTROL_BLOCK; var sv: TServerVar);

procedure Register_WABD_Callback(Callback: TWABD_Callback);


exports
   GetExtensionVersion,
   HttpExtensionProc;


implementation

uses TraceUnit, WABD_Objects;

var
   RunLocal       : boolean;
   WABD_Callback  : TWABD_Callback;


procedure Register_WABD_Callback(Callback: TWABD_Callback);
begin
   WABD_Callback := Callback;
end;

function RunningLocal: boolean;
begin
   Result := RunLocal;
end;

// ****************************************************************
// ISAPI Interface Functions

function GetExtensionVersion(var Ver: THSE_VERSION_INFO):boolean; stdcall;
begin
  Ver.dwExtensionVersion := $00010000;                // 1.0 support
  Ver.lpszExtensionDesc  := 'Delphi 2.0 ISAPI DLL';   // Description
  Result := True;
end;

function HttpExtensionProc( var ECB: TEXTENSION_CONTROL_BLOCK ):DWORD; stdcall;
const
   CR = #13#10;
   TENMINUTES = 10.0 / (24.0 * 60.0);
var
  ResStr : string;
  StrLen : integer;
  Start  : longint;
begin
   if ECB.dwVersion <> $DEADF00D then RunLocal := False;

   Start := GetTickCount;
   TRACE('ConnID = %8.8x', [ECB.ConnID]);
   ECB.lpszLogData      := 'Delphi DLL Log';
   ECB.dwHTTPStatusCode := 200;

   if Assigned(WABD_Callback) then ResStr := WABD_Callback(ECB)
      else begin
         ResStr := 'WABD Callback not registered!';
         LocalBrowserException := ResStr;
      end;

   ResStr := Format(
      'HTTP/1.0 200 OK'+CR+
      // 'Expires: ' + FormatDateTime('ddd, dd mmm yyyy hh:nn:ss EST', Now + TENMINUTES) + CR +
      'Content-Type: text/html'+CR+
      'Content-Length: %d'+CR+
      'Content:'+CR+CR, [Length(ResStr)]) + ResStr;
   StrLen := Length(ResStr);
   ECB.WriteClient(ECB.ConnID, @ResStr[1], StrLen, 0);
   Result := HSE_STATUS_SUCCESS;
   TRACE('Completed %8.8x (HTML Length = %d) Time = %6.2f sec',
      [ECB.ConnID, Length(ResStr), (GetTickCount-Start)/1000.0]);
   TRACE0('');
end;


// ****************************************************************
// ISAPI Helper Functions

function  ISAPI_GetOneVar(var ECB: TEXTENSION_CONTROL_BLOCK; const VarName: string): string;
var
  StrLen : integer;
  Buf    : array[0..1024] of char;
begin
   StrLen := Sizeof(Buf);
   ECB.GetServerVariable(ECB.ConnID, PChar(VarName), @Buf, StrLen);
   Result := Buf;
   // TRACE('Server Variarble %-16s = %s', [VarName, Buf]);
end;


procedure ISAPI_GetServerVars(var ECB: TEXTENSION_CONTROL_BLOCK; var sv: TServerVar);
begin
   // TRACE('Path Info = %s', [ECB.lpszPathInfo]);
   // TRACE('Path Translaged = %s', [ECB.lpszPathTranslated]);
   // TRACE('Query String = %s', [ECB.lpszQueryString]);
   sv.REMOTE_ADDR     := ISAPI_GetOneVar(ECB, 'REMOTE_ADDR');
   sv.REMOTE_HOST     := ISAPI_GetOneVar(ECB, 'REMOTE_HOST');
   sv.REMOTE_USER     := ISAPI_GetOneVar(ECB, 'REMOTE_USER');
   sv.SERVER_NAME     := ISAPI_GetOneVar(ECB, 'SERVER_NAME');
   sv.SERVER_PORT     := ISAPI_GetOneVar(ECB, 'SERVER_PORT');
   sv.SERVER_PROTOCOL := ISAPI_GetOneVar(ECB, 'SERVER_PROTOCOL');
   sv.SERVER_SOFTWARE := ISAPI_GetOneVar(ECB, 'SERVER_SOFTWARE');
   sv.SCRIPT_NAME     := ISAPI_GetOneVar(ECB, 'SCRIPT_NAME');
end;


initialization
   RunLocal       := True;
   WABD_Callback  := nil;
end.

